@@ -5,7 +5,7 @@ use base qw( Net::LDAP::Class::Group );
 use Carp;
 use Data::Dump ();
 
-our $VERSION = '0.18_03';
+our $VERSION = '0.04';
 
 =head1 NAME
 
@@ -17,8 +17,8 @@ Net::LDAP::Class::Group::AD - Active Directory group class
  package MyLDAPGroup;
  use base qw( Net::LDAP::Class::Group::AD );
  
- __PACKAGE__->metadata->setup(
-    base_dn             => 'dc=mycompany,dc=com',
+ __PACKAGE__->meta->setup(
+     base_dn             => 'dc=mycompany,dc=com',
     attributes          => __PACKAGE__->AD_attributes,
     unique_attributes   => __PACKAGE__->AD_unique_attributes,
  );
@@ -56,7 +56,6 @@ sub AD_attributes {
             canonicalName
             cn
             description
-            distinguishedName
             info
             member
             primaryGroupToken
@@ -75,7 +74,7 @@ Returns array ref of unique Active Directory attributes.
 =cut
 
 sub AD_unique_attributes {
-    [qw( cn objectSID distinguishedName )];
+    [qw( cn objectSID )];
 }
 
 =head1 OBJECT METHODS
@@ -103,10 +102,9 @@ sub fetch_primary_users {
     my $user_class = $self->user_class;
     my $pgt        = $self->primaryGroupToken;
     my @users      = $user_class->find(
-        scope   => 'sub',
-        filter  => "(primaryGroupID=$pgt)",
-        ldap    => $self->ldap,
-        base_dn => $self->base_dn,
+        scope  => 'sub',
+        filter => "(primaryGroupID=$pgt)",
+        ldap   => $self->ldap,
     );
 
     return wantarray ? @users : \@users;
@@ -131,19 +129,8 @@ sub fetch_secondary_users {
     my @users;
     for my $dn (@members) {
         my ($cn) = ( $dn =~ m/^cn=([^,]+),/i );
-        $dn =~ s/\(/\\(/g;
-        $dn =~ s/\)/\\)/g;
-        my $user = $user_class->new(
-            distinguishedName => $dn,
-            ldap              => $self->ldap,
-            base_dn           => $self->base_dn,
-        )->read;
-        if ($user) {
-            push( @users, $user );
-        }
-        else {
-            croak "can't find user $cn ($dn) via LDAP";
-        }
+        my $user = $user_class->new( cn => $cn, ldap => $self->ldap )->read;
+        push( @users, $user );
     }
     return wantarray ? @users : \@users;
 }
@@ -174,7 +161,7 @@ sub action_for_create {
 
     my @actions = (
         add => [
-            {   dn   => "CN=$name," . $self->base_dn,
+            {   dn   => "CN=$name,CN=Users," . $self->base_dn,
                 attr => [
                     objectClass => [ 'top', 'group' ],
                     cn          => $name,
@@ -197,65 +184,13 @@ sub action_for_update {
     my $self = shift;
     my %opts = @_;
 
-    my $base_dn = delete $opts{base_dn} || $self->base_dn;
+    if ( !grep { exists $self->{_was_set}->{$_} } @{ $self->attributes } ) {
+        warn "no attributes have changed for group $self. Skipping update().";
+        return 1;
+    }
 
     my @actions;
 
-    # users get translated to 'member' attribute
-    if ( exists $self->{users} ) {
-
-        my @names;
-        for my $user ( @{ delete $self->{users} } ) {
-            my $dn = $user->ldap_entry->dn;
-            push @names, $dn;
-        }
-        $self->member( \@names );    # should trigger _was_set below
-
-    }
-
-    # which fields have changed.
-    my %replace;
-    for my $attr ( keys %{ $self->{_was_set} } ) {
-
-        next if $attr eq 'cn';                   # part of DN
-        next if $attr eq 'objectSID';            # set by server
-        next if $attr eq 'primaryGroupToken';    # set by server
-
-        my $old = $self->{_was_set}->{$attr}->{old};
-        my $new = $self->{_was_set}->{$attr}->{new};
-
-        if ( defined($old) and !defined($new) ) {
-            $replace{$attr} = undef;
-        }
-        elsif ( !defined($old) and defined($new) ) {
-            $replace{$attr} = $new;
-        }
-        elsif ( !defined($old) and !defined($new) ) {
-
-            #$replace{$attr} = undef;
-        }
-        elsif ( $old ne $new ) {
-            $replace{$attr} = $new;
-        }
-
-    }
-
-    if (%replace) {
-        my $cn = $self->name;
-        push(
-            @actions,
-            update => {
-                search => [
-                    base   => $base_dn,
-                    scope  => "sub",
-                    filter => "(cn=$cn)",
-                    attrs  => $self->attributes,
-                ],
-                replace => \%replace
-            }
-        );
-    }
-
     if ( exists $self->{_was_set}->{cn} ) {
 
         my $class = ref($self) || $self;
@@ -270,6 +205,8 @@ sub action_for_update {
             = $class->new( ldap => $self->ldap, cn => $old_name )->read
             or croak "can't find $old_name in LDAP";
 
+        # TODO must be change 'memberOf' attributes for all related users?
+
         # two steps since cn is part of the dn.
         # first, create a new group with the new name
         push( @actions, $self->action_for_create( cn => $new_name ) );
@@ -279,11 +216,6 @@ sub action_for_update {
 
     }
 
-    if ( !@actions ) {
-        warn "no attributes have changed for group $self. Skipping update().";
-        return @actions;
-    }
-
     return @actions;
 }
 
@@ -307,10 +239,7 @@ sub action_for_delete {
 
     # even if called a class method, we need an object
     # in order to find users, etc.
-    my $group
-        = ref($self)
-        ? $self
-        : $self->new( cn => $name, ldap => $self->ldap )->read;
+    my $group = ref($self) ? $self : $self->new( cn => $name )->read;
     if ( !$group ) {
         croak "no such Group to delete: $name";
     }
@@ -319,10 +248,10 @@ sub action_for_delete {
 
     my @actions = (
         {   search => [
-                base   => $group->base_dn,
+                base   => 'CN=Users,' . $group->base_dn,
                 scope  => 'sub',
                 filter => "(cn=$name)",
-                attrs  => $group->attributes,
+                attrs  => $group->meta->attributes,
             ],
         }
     );
@@ -330,60 +259,6 @@ sub action_for_delete {
     return ( delete => \@actions );
 }
 
-=head2 add_user( I<user_object> )
-
-Push I<user_object> onto the list of member() DNs, checking
-that I<user_object> is not already on the list.
-
-=cut
-
-sub add_user {
-    my $self = shift;
-    my $user = shift;
-    if ( !$user or !ref($user) or !$user->isa('Net::LDAP::Class::User::AD') )
-    {
-        croak "Net::LDAP::Class::User::AD object required";
-    }
-    unless ( $user->username ) {
-        croak
-            "User object must have at least a username before adding to group $self";
-    }
-    for my $u ( $self->secondary_users ) {
-        if ( "$u" eq "$user" ) {
-            croak "User $user is already a member of group $self";
-        }
-    }
-    my @users = $self->secondary_users;
-    push( @users, $user );
-    $self->{users} = \@users;
-}
-
-=head2 remove_user( I<user_object> )
-
-Drop I<user_object> from the list of member() DNs, checking
-that I<user_object> is already on the list.
-
-=cut
-
-sub remove_user {
-    my $self = shift;
-    my $user = shift;
-    if ( !$user or !ref($user) or !$user->isa('Net::LDAP::Class::User::AD') )
-    {
-        croak "Net::LDAP::Class::User::AD object required";
-    }
-    unless ( $user->username ) {
-        croak
-            "User object must have at least a username before removing from group $self";
-    }
-    my %users = map { $_->username => $_ } @{ $self->secondary_users };
-    if ( !exists $users{ $user->username } ) {
-        croak "User $user is not a member of group $self";
-    }
-    delete $users{ $user->username };
-    $self->{users} = [ values %users ];
-}
-
 1;
 
 __END__
