@@ -4,12 +4,13 @@ use warnings;
 use base qw( Net::LDAP::Class::User );
 use Carp;
 use Data::Dump ();
+use Encode;
 
-use Net::LDAP::Class::MethodMaker (
+use Rose::Object::MakeMethods::Generic (
     'scalar --get_set_init' => [qw( default_home_dir default_email_suffix )],
 );
 
-our $VERSION = '0.18_03';
+our $VERSION = '0.04';
 
 =head1 NAME
 
@@ -21,7 +22,7 @@ Net::LDAP::Class::User::AD - Active Directory User class
  package MyLDAPUser;
  use base qw( Net::LDAP::Class::User::AD );
  
- __PACKAGE__->metadata->setup(
+ __PACKAGE__->meta->setup(
     base_dn             => 'dc=mycompany,dc=com',
     attributes          => __PACKAGE__->AD_attributes,
     unique_attributes   => __PACKAGE__->AD_unique_attributes,
@@ -62,30 +63,20 @@ similarity to the POSIX schema.
 sub AD_attributes {
     [   qw(
             accountExpires
-            adminCount
             canonicalName
             cn
-            codePage
-            countryCode
             description
             displayName
-            distinguishedName
             givenName
             groupAttributes
             homeDirectory
             homeDrive
-            instanceType
-            lastLogoff
-            lastLogon
-            logonCount
             mail
             memberOf
             middleName
             modifyTimeStamp
-            name
             notes
             objectClass
-            objectGUID
             objectSID
             primaryGroupID
             profilePath
@@ -95,12 +86,6 @@ sub AD_attributes {
             sn
             uid
             unicodePwd
-            userAccountControl
-            userPrincipalName
-            uSNChanged
-            uSNCreated
-            whenCreated
-            whenChanged
             )
     ];
 }
@@ -112,7 +97,7 @@ Returns array ref of unique Active Directory attributes.
 =cut
 
 sub AD_unique_attributes {
-    [qw( sAMAccountName distinguishedName objectSID )];
+    [qw( sAMAccountName cn objectSID )];
 }
 
 =head1 OBJECT METHODS
@@ -131,7 +116,7 @@ Returns an object of type group_class().
 sub fetch_group {
     my $self  = shift;
     my $class = $self->group_class or croak "group_class() required";
-    my $gid   = shift || $self->gid;
+    my $gid   = shift || $self->primaryGroupID;
 
     if ( !$gid ) {
         croak "cannot fetch group without a gid (primaryGroupID) set";
@@ -139,56 +124,26 @@ sub fetch_group {
 
     # because AD does not store primaryGroupToken but computes it,
     # we must do gymnastics using SIDs
-    warn "gid = $gid";
+    #warn "gid = $gid";
+    my (@unpack) = unpack( "H2 H2 n N V*", $self->objectSID );
+    my ( $sid_rev, $num_auths, $id1, $id2, @ids ) = (@unpack);
 
-    my $user_sid_string = _sid2string( $self->objectSID );
+    #warn Data::Dump::dump \@unpack;
+    my $user_sid_string
+        = join( "-", "S", $sid_rev, ( $id1 << 32 ) + $id2, @ids );
 
-    warn "user_sid_string:  $user_sid_string";
+    #warn "user_sid_string:  $user_sid_string";
     ( my $group_sid_string = $user_sid_string ) =~ s/\-[^\-]+$/-$gid/;
 
-    warn "group_sid_string: $group_sid_string";
+    #warn "group_sid_string: $group_sid_string";
 
+    # get groups too
     return $class->new(
         objectSID => $group_sid_string,
         ldap      => $self->ldap
     )->read;
 }
 
-sub _sid2string {
-    my $sid = shift;
-    carp "nlc sid    = " . Data::Dump::dump($sid);
-    my (@unpack) = unpack( "H2 H2 n N V*", $sid );
-    my ( $sid_rev, $num_auths, $id1, $id2, @ids ) = (@unpack);
-    my $string = join( "-", "S", $sid_rev, ( $id1 << 32 ) + $id2, @ids );
-    carp "nlc string = $string";
-    return $string;
-}
-
-sub _string2sid {
-    my $string = shift;
-    my (@split) = split( m/\-/, $string );
-    my ( $prefix, $sid_rev, $auth_id, @ids ) = (@split);
-    if ( $auth_id != scalar(@ids) ) {
-        die "bad string: $string";
-    }
-
-    my $sid = pack( "C4", "$sid_rev", "$auth_id", 0, 0 );
-    $sid .= pack( "C4",
-        ( $auth_id & 0xff000000 ) >> 24,
-        ( $auth_id & 0x00ff0000 ) >> 16,
-        ( $auth_id & 0x0000ff00 ) >> 8,
-        $auth_id & 0x000000ff );
-
-    for my $i (@ids) {
-        $sid .= pack( "I", $i );
-    }
-    
-    carp "nlc string = $string";
-    carp "nlc sid    = " . Data::Dump::dump($sid);
-
-    return $sid;
-}
-
 =head2 fetch_groups
 
 Required MethodMaker method for retrieving secondary groups from LDAP.
@@ -199,25 +154,20 @@ group_class().
 =cut
 
 sub fetch_groups {
-    my $self = shift;
+    my $self        = shift;
+    my @group_dns   = $self->memberOf;
+    my $group_class = $self->group_class;
     my @groups;
-
-    if ( $self->ldap_entry ) {
-        my @group_dns   = $self->ldap_entry->get_value('memberOf');
-        my $group_class = $self->group_class;
-
-        for my $dn (@group_dns) {
-            $dn =~ s/^cn=([^,]+),.+/$1/i;
-            push(
-                @groups,
-                $group_class->new(
-                    cn   => $dn,
-                    ldap => $self->ldap
-                    )->read
-            );
-        }
+    for my $dn (@group_dns) {
+        $dn =~ s/^cn=([^,]+),.+/$1/i;
+        push(
+            @groups,
+            $group_class->new(
+                cn   => $dn,
+                ldap => $self->ldap
+                )->read
+        );
     }
-
     return wantarray ? @groups : \@groups;
 }
 
@@ -296,12 +246,11 @@ sub password {
     return $self->ldap_entry->get_value($attribute);
 }
 
-sub _is_encoded {
+sub _is_utf16 {
     my $str = shift;
-    if ( $str =~ m/^"\000.+"\000$/ ) {
-        return 1;
-    }
-    return 0;
+    my $octets;
+    eval { $octets = encode( "UTF-16", $str, 1 ); };
+    return $@ ? 0 : 1;
 }
 
 sub _encode_pass {
@@ -309,33 +258,14 @@ sub _encode_pass {
     my $pass = shift or croak "password required";
 
     # detect if password is already encoded and do not double encode
-    if ( _is_encoded($pass) ) {
+    if ( $pass =~ m/^"/ and _is_utf16($pass) ) {
         return $pass;
     }
 
-    my $npass = '';
-    map { $npass .= "$_\000" } split( //, "\"$pass\"" );
-
+    my $npass = encode( "UTF-16", "\"$pass\"", 1 );
     return $npass;
 }
 
-sub _decode_pass {
-    my $self = shift;
-    my $pass = shift or croak "password required";
-    if ( !_is_encoded($pass) ) {
-        return $pass;
-    }
-
-    my $decoded = '';
-    for my $char ( split( //, $pass ) ) {
-        $char =~ s/\000$//;
-        $decoded .= $char;
-    }
-    $decoded =~ s/^"|"$//g;
-
-    return $decoded;
-}
-
 =head2 action_for_create([ sAMAccountName => I<username> ])
 
 Returns hash ref suitable for creating a Net::LDAP::Batch::Action::Add.
@@ -350,102 +280,39 @@ sub action_for_create {
     my %opts     = @_;
     my $username = delete $opts{sAMAccountName} || $self->sAMAccountName
         or croak "sAMAccountName required to create()";
-    my $base_dn = delete $opts{base_dn} || $self->base_dn;
 
-    my ( $group, $gid, $givenName, $sn, $cn, $email )
+    my ( $group, $gid, $givenName, $sn, $cn, $email, $pass )
         = $self->setup_for_write;
 
-    #warn "AD setup_for_write() $base_dn";
-
-    my $pass = $self->password || $self->random_string(10);
-    $pass = $self->_encode_pass($pass);
-
-# see
-# http://www.sysoptools.com/support/files/Fixing%20user%20accounts%20flagged%20as%20system%20accounts%20-%20the%20UserAccountControl%20AD%20attribute.doc
-# for details on userAccountControl.
-# basically:
-#  512 - normal active account requiring password
-#  514 - normal disabled account requiring password
-#  544 - system active account - no password required
-#  546 - system disabled account - no password required (default)
-
-    my %attr = (
-        objectClass => [ "top", "person", "organizationalPerson", "user" ],
-        sAMAccountName => $username,
-        givenName      => $givenName,
-        displayName    => $cn,
-        sn             => $sn,
-        cn             => $cn,
-        homeDirectory  => $self->default_home_dir . "\\$username",
-        mail           => $email,
-        userAccountControl => 512,     # so AD treats it as a Normal user
-        unicodePwd         => $pass,
-    );
-
-    $attr{primaryGroupID} = $gid if $gid;
-
-    # mix in whatever has been set
-    for my $name ( keys %{ $self->{_not_yet_set} } ) {
-
-        #warn "set $name => $self->{_not_yet_set}->{$name}";
-        unless ( exists $attr{$name} ) {
-            $attr{$name} = delete $self->{_not_yet_set}->{$name};
-        }
-        else {
-            $attr{$name} = $self->{_not_yet_set}->{$name};
-        }
-    }
-
-    my $dn = "CN=$cn,$base_dn";
-
     my @actions = (
         add => {
-            dn   => $dn,
-            attr => [%attr]
+            dn   => "CN=$cn,CN=Users," . $self->base_dn,
+            attr => [
+                objectClass =>
+                    [ "top", "person", "organizationalPerson", "user" ],
+                sAMAccountName => $username,
+                givenName      => $givenName,
+                displayName    => $cn,
+                sn             => $sn,
+                cn             => $cn,
+                homeDirectory  => $self->default_home_dir . "\\$username",
+                mail           => $email,
+                unicodePwd     => $pass,
+            ],
         }
     );
 
-    #warn "AD checking groups $base_dn";
+    push( @{ $actions[1]->{attr} }, primaryGroupID => $gid ) if $gid;
 
     # groups
     if ( exists $self->{groups} ) {
-
-        #carp $self->dump;
-
-        #warn "User $self has groups assigned";
-        #warn Data::Dump::dump $self->{groups};
-
-    G: for my $group ( @{ $self->{groups} } ) {
-            if ( !$group->ldap_entry ) {
-                croak
-                    "You must create group $group before you add User $self to it";
-            }
-
-            #warn "checking if $group has user $self";
-
-            # only interested in new additions
-            next G if $group->has_user($self);
-
-            #warn "group $group does not yet have user $self";
-
-            my $group_cn = $group->cn;
-            my @members  = $group->member;
-            push( @members, $dn );
-
-            push(
-                @actions,
-                update => {
-                    search => [
-                        base   => $group->base_dn,
-                        scope  => "sub",
-                        filter => "(cn=$group_cn)",
-                        attrs  => $group->attributes,
-                    ],
-                    replace => { member => \@members },
-                }
-            );
-
+        my @names;
+        for my $group ( @{ $self->{groups} } ) {
+            my $group_name = $group->name;
+            my $group_dn   = $group->base_dn;
+            push( @names, "CN=$group_name,CN=Users,$group_dn" );
         }
+        push( @{ $actions[1]->{attr} }, memberOf => \@names );
     }
 
     return @actions;
@@ -459,7 +326,7 @@ and action_for_update().
 
 Returns array of values in this order:
 
- $groupname, $gid, $givenName, $sn, $cn, $email
+ $groupname, $gid, $givenName, $sn, $cn, $email, $password
 
 =cut
 
@@ -467,15 +334,15 @@ sub setup_for_write {
     my $self = shift;
 
     my $gid;
-    my $group = $self->{group} || $self->gid;
+    my $group = $self->{group} || $self->primaryGroupID;
     if ($group) {
-        if ( ref $group and $group->isa('Net::LDAP::Class::Group') ) {
+        if ( ref $group and $group->isa('Net::LDAP::Class::Group::AD') ) {
             $gid = $group->gid;
         }
         else {
             my $group_obj = $self->fetch_group($group);
             if ( !$group_obj ) {
-                confess "no such group in AD server: $group";
+                croak "no such group in AD server: $group";
             }
             $gid = $group_obj->gid;
         }
@@ -492,19 +359,18 @@ sub setup_for_write {
 
     # the name logic breaks horribly here for anything but trivial cases.
     my @name_parts = split( m/\s+/, $self->cn || $self->displayName || '' );
+    my $givenName = $self->givenName || shift(@name_parts);
+    my $sn        = $self->sn        || join( ' ', @name_parts );
+    my $cn        = $self->cn        || join( ' ', $givenName, $sn );
 
-    my $givenName = $self->givenName;
-    $givenName = shift(@name_parts) unless defined $givenName;
-    my $sn = $self->sn;
-    $sn = join( ' ', @name_parts ) unless defined $sn;
-    my $cn = $self->cn;
-    $cn = join( ' ', $givenName, $sn ) unless defined $cn;
+    my $email = $self->mail || $self->username . $self->default_email_suffix;
 
-    my $email = $self->mail;
-    $email = ( $self->username . $self->default_email_suffix )
-        unless defined $email;
+    # set password if not set.
+    # this is useful for default random passwords.
+    my $pass = $self->unicodePwd || $self->random_string(10);
+    $pass = $self->_encode_pass($pass);
 
-    return ( $group, $gid, $givenName, $sn, $cn, $email );
+    return ( $group, $gid, $givenName, $sn, $cn, $email, $pass );
 }
 
 =head2 action_for_update
@@ -515,15 +381,13 @@ Returns array ref suitable for creating a Net::LDAP::Batch::Action::Update.
 
 sub action_for_update {
     my $self     = shift;
-    my %opts     = @_;
+    my %opts     = @_;                # currently unused
     my $username = $self->username;
 
     unless ($username) {
         croak "must have sAMAccountName set to update";
     }
 
-    my $base_dn = delete $opts{base_dn} || $self->base_dn;
-
     my @actions;
 
     my ( $group, $gid, $givenName, $sn, $cn, $email, $pass )
@@ -584,74 +448,14 @@ sub action_for_update {
     # check if any have been set explicitly,
     # since otherwise there is nothing to be done.
     if ( exists $self->{groups} ) {
-
-        #carp Data::Dump::dump $self->{groups};
-
-        my $existing_groups = $self->fetch_groups;
-
-        #carp Data::Dump::dump $existing_groups;
-
-        my %existing = map { $_->cn => $_ } @$existing_groups;
-
-        # the delete $self->{groups} has helpful side effect of clearing
-        # cache.
-        my %new = map { $_->cn => $_ } @{ delete $self->{groups} };
-
-        #warn "User $self has " . scalar( keys %new ) . " groups set";
-        #warn "existing group: $_" for sort keys %existing;
-        #warn "new group     : $_" for sort keys %new;
-
-        # which should be added
-        my @to_add;
-    G: for my $cn ( keys %new ) {
-            if ( !exists $existing{$cn} ) {
-                my $group = $new{$cn};
-
-                if ( !$group->ldap_entry ) {
-                    croak(
-                        "you must create $group before adding user $self to it"
-                    );
-                }
-
-                for my $u ( $group->secondary_users ) {
-
-                    #warn " group member: $u <> user $self";
-
-                    next G if "$u" eq "$self";
-
-                }
-
-                #warn "group $group does NOT have User $self assigned";
-                $group->add_user($self);
-
-                push( @to_add, $group->action_for_update );
-
-            }
+        my @names;
+        for my $group ( @{ $self->{groups} } ) {
+            my $group_name = $group->name;
+            my $group_dn   = $group->base_dn;
+            push( @names, "CN=$group_name,CN=Users,$group_dn" );
         }
-
-        # which should be removed
-        my @to_rm;
-    G: for my $cn ( keys %existing ) {
-            if ( !exists $new{$cn} ) {
-                my $group = $existing{$cn};
-
-                #next unless $group->has_user($self);
-
-                for my $u ( $group->secondary_users ) {
-                    next G unless "$u" eq "$self";
-                }
-
-                #warn "group $group does have User $self assigned";
-
-                $group->remove_user($self);
-
-                push( @to_rm, $group->action_for_update );
-
-            }
-        }
-
-        push( @actions, @to_add, @to_rm );
-
+        delete $self->{groups};
+        $replace{memberOf} = \@names;
     }
 
     if (%replace) {
@@ -659,10 +463,10 @@ sub action_for_update {
             @actions,
             update => {
                 search => [
-                    base   => $base_dn,
+                    base   => "CN=Users," . $self->base_dn,
                     scope  => "sub",
                     filter => "(sAMAccountName=$username)",
-                    attrs  => $self->attributes,
+                    attrs  => $self->meta->attributes,
                 ],
                 replace => \%replace
             }
@@ -683,7 +487,7 @@ sub action_for_update {
 
 =head2 action_for_delete
 
-Returns action suitable for creating a Net::LDAP::Batch::Action::Delete.
+Returns hash ref suitable for creating a Net::LDAP::Batch::Action::Delete.
 
 =cut
 
@@ -694,8 +498,6 @@ sub action_for_delete {
         || delete $opts{username}
         || $self->username;
 
-    my $base_dn = delete $opts{base_dn} || $self->base_dn;
-
     if ( !$username ) {
         croak "username required to delete a User";
     }
@@ -704,10 +506,10 @@ sub action_for_delete {
     my @actions = (
         delete => {
             search => [
-                base   => $base_dn,
+                base   => "CN=Users," . $self->base_dn,
                 scope  => "sub",
                 filter => "(sAMAccountName=$username)",
-                attrs  => $self->attributes,
+                attrs  => $self->meta->attributes,
             ]
         }
     );
